#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2011,2013 Ivan Kluchnikov, Alexander Chemeris
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
# USA

from socket import socket, AF_INET, SOCK_DGRAM
import SocketServer
import threading
import sys
import struct
import time
import re

transceiver_time = -1
last_tx_time = -1

tn = 0
power_att = 0
tx_delay = 20 # in frames

regex_ind_clk = re.compile("IND CLOCK (\d+)")
our_host = '127.0.0.1'
trx_host = '127.0.0.1'
trx_clk_port = 5700
our_clk_port = trx_clk_port+100
trx_cmd_port = trx_clk_port+1
our_cmd_port = our_clk_port+1
trx_data_port = our_clk_port+2

dummy_burst = (
  0,0,0,
  1,1,1,1,1,0,1,1,0,1,1,1,0,1,1,0,0,0,0,0,1,0,1,0,0,1,0,0,1,1,1,0,
  0,0,0,0,1,0,0,1,0,0,0,1,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,1,1,1,0,0,
  0,1,0,1,1,1,0,0,0,1,0,1,1,1,0,0,0,1,0,1,0,1,1,1,0,1,0,0,1,0,1,0,
  0,0,1,1,0,0,1,1,0,0,1,1,1,0,0,1,1,1,1,0,1,0,0,1,1,1,1,1,0,0,0,1,
  0,0,1,0,1,1,1,1,1,0,1,0,1,0,
  0,0,0
)

normal_burst = (
  0,0,0,
  1,1,0,1,1,1,0,1,0,1,0,1,1,0,1,0,1,0,1,0,0,1,0,0,1,1,0,1,1,0,1,0,
  1,1,0,1,1,1,0,1,0,1,1,1,0,0,1,0,1,1,0,0,0,1,0,0,0,0,0,0,1,0,0,0,
  0,0,0,1,1,1,1,0,0,1,0,1,1,0,1,0,1,0,0,0,1,0,1,0,1,1,1,0,1,1,0,0,
  0,0,1,1,0,1,1,0,0,1,0,1,1,1,1,1,0,1,0,1,0,0,0,1,1,1,1,1,0,0,1,0,
  1,0,1,0,1,1,0,1,1,1,1,1,1,1,
  0,0,0
)

def send_cmd_packet(trx, udp, packet):
  udp.sendto(packet+'\0', (trx_host, trx_cmd_port+trx*2))
  print("CMD sent : %s" % (packet))
  data, addr = udp.recvfrom(1024)
  print("CMD response from %s:%s : %s" % (addr[0], addr[1], data.strip()))

def send_data(burst, fn, tn, pwr_att):
  udp = socket(AF_INET,SOCK_DGRAM)
  # Generate header
  header = struct.pack('b',tn) # TN
  header = header + struct.pack('!i', fn) # FN
  header = header + struct.pack('b', pwr_att) # power attenuation
  # Generate data
  data = ''
  for d in dummy_burst:
    data = data + struct.pack('b', d)
  # Send packet
  udp.sendto(header+data, (trx_host, trx_data_port))
  print("DATA pkt sent @ TN=%d FN=%d pwr_att=%d" % (tn, fn, pwr_att))
#  print("header sent : %s" % (struct.unpack('bbbbbb',header),))
#  print("DATA sent : %s" % (burst,))

class ClkIndHandler(SocketServer.BaseRequestHandler):
  """
  """

  def handle(self):
    global last_tx_time
    global transceiver_time
    data = self.request[0].strip()
    socket = self.request[1]
    print("Received: %s" % data)
    transceiver_time = int(regex_ind_clk.search(data).group(1))
    print("Transceiver time: %d (last_tx_time: %d)" % (transceiver_time, last_tx_time))
    # Transmit pending data
    if last_tx_time == -1:
      last_tx_time = transceiver_time + tx_delay
      send_data(normal_burst, last_tx_time, 0, 0)
    while last_tx_time < transceiver_time + tx_delay:
      last_tx_time = last_tx_time + 1
      send_data(normal_burst, last_tx_time, 0, 0)


class DataRxHandler(SocketServer.BaseRequestHandler):
  """
  """

  def handle(self):
    data = self.request[0].strip()
    socket = self.request[1]
    print("Received: %s" % data)


if __name__ == "__main__":
  cmd_sock = socket(AF_INET,SOCK_DGRAM)
  cmd_sock.settimeout(1.0)
  cmd_sock.bind((trx_host, our_cmd_port))
  cmd_sock2 = socket(AF_INET,SOCK_DGRAM)
  cmd_sock2.settimeout(1.0)
  cmd_sock2.bind((trx_host, our_cmd_port+2))


  send_cmd_packet(0, cmd_sock, 'CMD POWEROFF')
  send_cmd_packet(0, cmd_sock, 'CMD RXTUNE 904800')
  send_cmd_packet(0, cmd_sock, 'CMD TXTUNE 1877600')
#  send_cmd_packet(0, cmd_sock, 'CMD SETMAXDLY 35')
  send_cmd_packet(0, cmd_sock, 'CMD SETTSC 0')
  send_cmd_packet(0, cmd_sock, 'CMD POWERON')
  send_cmd_packet(0, cmd_sock, 'CMD SETRXGAIN 0')
  send_cmd_packet(0, cmd_sock, 'CMD SETPOWER 0')
  send_cmd_packet(0, cmd_sock, 'CMD SETSLOT 0 1')
  send_cmd_packet(0, cmd_sock, 'CMD SETSLOT 1 1')
  send_cmd_packet(0, cmd_sock, 'CMD SETSLOT 2 1')
  send_cmd_packet(0, cmd_sock, 'CMD SETSLOT 3 1')
  send_cmd_packet(0, cmd_sock, 'CMD SETSLOT 4 1')
  send_cmd_packet(0, cmd_sock, 'CMD SETSLOT 5 1')
  send_cmd_packet(0, cmd_sock, 'CMD SETSLOT 6 1')
  send_cmd_packet(0, cmd_sock, 'CMD SETSLOT 7 1')


  send_cmd_packet(1, cmd_sock2, 'CMD POWEROFF')
  send_cmd_packet(1, cmd_sock2, 'CMD RXTUNE 904800')
  send_cmd_packet(1, cmd_sock2, 'CMD TXTUNE 1877600')
#  send_cmd_packet(1, cmd_sock2, 'CMD SETMAXDLY 35')
  send_cmd_packet(1, cmd_sock2, 'CMD SETTSC 0')
  send_cmd_packet(1, cmd_sock2, 'CMD POWERON')
  send_cmd_packet(1, cmd_sock2, 'CMD SETRXGAIN 0')
  send_cmd_packet(1, cmd_sock2, 'CMD SETPOWER 0')
  send_cmd_packet(1, cmd_sock2, 'CMD SETSLOT 0 1')
  send_cmd_packet(1, cmd_sock2, 'CMD SETSLOT 1 1')
  send_cmd_packet(1, cmd_sock2, 'CMD SETSLOT 2 1')
  send_cmd_packet(1, cmd_sock2, 'CMD SETSLOT 3 1')
  send_cmd_packet(1, cmd_sock2, 'CMD SETSLOT 4 1')
  send_cmd_packet(1, cmd_sock2, 'CMD SETSLOT 5 1')
  send_cmd_packet(1, cmd_sock2, 'CMD SETSLOT 6 1')
  send_cmd_packet(1, cmd_sock2, 'CMD SETSLOT 7 1')


  sys.exit(0)

  time.sleep(3)

  print('Starting CLK rx loop thread...')
  server_clk = SocketServer.UDPServer((our_host, our_clk_port), ClkIndHandler)
  server_clk_thread = threading.Thread(target=server_clk.serve_forever)
  # Exit the server thread when the main thread terminates
  server_clk_thread.daemon = True
  server_clk_thread.start()
  print("CLK rx loop running in thread: %s" % server_clk_thread.name)

  print('Starting DATA rx loop thread...')
  server_clk = SocketServer.UDPServer((our_host, our_data_port), DataRxHandler)
  server_clk_thread = threading.Thread(target=server_clk.serve_forever)
  # Exit the server thread when the main thread terminates
  server_clk_thread.daemon = True
  server_clk_thread.start()
  print("DATA rx loop running in thread: %s" % server_clk_thread.name)

  time.sleep(10)

  send_cmd_packet(0, cmd_sock, 'CMD POWEROFF')
  send_cmd_packet(1, cmd_sock2, 'CMD POWEROFF')
